﻿/*
 * fooRunes - fixes icecast stream metadata
 * Copyright (C) 2010  ed <tripflag@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License v2
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, refer to the following URL:
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;
using System.Net;

namespace foorunes {
    public partial class Form1 : Form {
        public Form1() {
            InitializeComponent();
        }
        bool busy;      // To stop racing conditions
        Timer t, td;    // Metadata and delay timers
        string trace;   // Most recent exception
        NotifyIcon ni;  // Tray icon

        /// <summary>
        /// Initializes the application at start
        /// </summary>
        private void Form1_Load(object sender, EventArgs e) {
            busy = false;
            t = new Timer();
            t.Interval = 500;
            td = new Timer();
            t.Tick += new EventHandler(t_Tick);
            td.Tick += new EventHandler(td_Tick);
            loadConfig();

            ni = new NotifyIcon();
            ni.Icon = this.Icon;
            ni.Visible = true;
            ni.Click += delegate(object oa, EventArgs ob) {
                this.Visible = !this.Visible;
                if (this.Visible)
                    this.WindowState =
                    FormWindowState.Normal;
            };
            q("Ready");

            Timer derp = new Timer();
            // This probably won't make sense tomorrow
            derp.Tick += delegate(object oa, EventArgs ob) {
                ((Timer)oa).Stop();
                this.Visible = !_hide.Checked;
            };
            derp.Interval = 500;
            derp.Start();
        }

        /// <summary>
        /// Starts/stops metareader timer
        /// </summary>
        private void _read_CheckedChanged(object sender, EventArgs e) {
            t.Enabled = _read.Checked;
        }

        /// <summary>
        /// Changes delay after metareader detects a change
        /// </summary>
        private void _don_TextChanged(object sender, EventArgs e) {
            try {
                td.Interval = Convert.ToInt32(_delay.Text);
            } catch { }
        }

        /// <summary>
        /// Extract metadata from Foobar2000
        /// </summary>
        string getFoo() {
            return getMeta("   [foobar2000 v", "foobar2000", 0);
        }

        /// <summary>
        /// Extract metadata from Winamp
        /// </summary>
        string getAmp() {
            string durr = getMeta(" - Winamp", "winamp", 0);
            if (string.IsNullOrEmpty(durr)) return null;

            // Trim tracknumber...
            int hurr = durr.IndexOf(". ");
            try {
                if (hurr > 0 && 0 < Convert.ToInt32(
                    durr.Substring(0, hurr))) durr =
                        durr.Substring(hurr + 2);
            } catch { }
            return durr;
        }

        /// <summary>
        /// Metadata extraction helper method
        /// </summary>
        /// <param name="trim">Suffix to crop away</param>
        /// <param name="app">Process title</param>
        /// <param name="n">Recursion depth</param>
        string getMeta(string trim, string app, int n) {
            try {
                Process[] proc = Process.GetProcessesByName(app);   // Find mediaplayer
                if (proc.Length != 1) return null;                  // Bail if none/multiple
                string cap = proc[0].MainWindowTitle;               // Extract window title
                int idx = cap.IndexOf(trim);                        // Find string to trim
                if (idx < 0) return null;                           // No match? Bail
                return cap.Substring(0, idx);                       // Return relevant part
            } catch {
                trace = "Could not find/parse from " + app;
                return null;
            }
        }

        /// <summary>
        /// Metadata reader
        /// </summary>
        void t_Tick(object sender, EventArgs e) {
            // Block race conditions
            if (busy) return;
            busy = true;

            string cap = null;
            int typ = _mp.SelectedIndex;
            switch (typ) {
                case 0: cap = getFoo(); break;
                case 1: cap = getAmp(); break;
            }
            if (!string.IsNullOrEmpty(cap)) {           // Got metadata?
                if (_meta.Text != cap) {                // Got new metadata?
                    _meta.Text = cap;
                    if (_sub.Checked) {                 // Autosubmit?
                        q("New metadata found");
                        if (!_delay.Checked) {          // Delay?
                            _do_Click(sender, e);       // No, instant
                        } else {                        // Yes, queue
                            td.Stop();
                            td.Start();
                        }
                    } else {                            // Write to np.txt
                        System.IO.File.WriteAllText(
                            "np.txt", _meta.Text,
                            Encoding.UTF8);
                    }
                }
            }
            busy = false;
        }
        void td_Tick(object sender, EventArgs e) {
            td.Stop(); _do_Click(sender, e);
            // Delayed submission
        }

        private void _do_Click(object sender, EventArgs e) {
            
            // Encode metadata for HTTP/GET submission
            string meta = System.Web.HttpUtility.UrlEncode(_meta.Text);
            string auth = Convert.ToBase64String(Encoding.ASCII.GetBytes(_pwd.Text));
            // ...also get the base64 authentication http header

            try {
                q("Metadata -> Server");
                using (WebClient wc = new WebClient()) {
                    if (_mnt.Text != "") {

                        // iceCast server
                        wc.Headers.Add("Authorization: Basic " + auth);
                        wc.DownloadString(
                            "http://" + _host.Text + "/admin/" +
                            "metadata?mode=updinfo&mount=" + _mnt.Text +
                            "&charset=UTF-8&song=" + meta);
                    } else {

                        // ShoutCast server
                        // WHAT THE FUCK ARE YOU DOING NULLSOFT
                        // I HATE YOU EVEN MORE NOW HOLY SHIT

                        //wc.DownloadString(
                        //    "http://" + _host.Text + "/admin.cgi?" +
                        //    "mode=updinfo&song=" + meta);
                        meta = meta.Replace("+", "%20");
                        byte[] buf = Encoding.UTF8.GetBytes(
                            "GET /admin.cgi?mode=updinfo&charset=UTF-8&song=" + meta + " HTTP/1.1" + "\r\n" +
                            "Host: h.faghats.com:9001" + "\r\n" + //This is understandable, but
                            "User-Agent: Mozilla/5.0" + "\r\n" + //WHY IS THIS NECESSARY DAMNIT
                            "Authorization: Basic " + auth + "\r\n\r\n"); //fucking trolls

                        // Create new socket, connect, send buffer, close connection.
                        System.Net.Sockets.Socket s = new System.Net.Sockets.Socket(
                            System.Net.Sockets.AddressFamily.InterNetwork,
                            System.Net.Sockets.SocketType.Stream,
                            System.Net.Sockets.ProtocolType.Tcp);
                        string[] host = _host.Text.Split(':');
                        s.Connect(host[0], Convert.ToInt32(host[1]));
                        s.Send(buf);
                        s.Close();
                    }
                }
            } catch (Exception ex) {
                trace = ex.Source + ": " + ex.Message + "\r\n\r\n" + ex.StackTrace;
                q("Failed to submit metadata");
            }
        }

        /// <summary>
        /// Load stored config (if possible)
        /// </summary>
        void loadConfig() {
            try {
                string[] conf = System.IO.File.ReadAllLines("foorunes.conf", Encoding.UTF8);
                _hide.Checked = Convert.ToBoolean(conf[0]);
                _read.Checked = Convert.ToBoolean(conf[1]);
                _mp.SelectedIndex = Convert.ToInt32(conf[2]);
                _sub.Checked = Convert.ToBoolean(conf[3]);
                _delay.Checked = Convert.ToBoolean(conf[4]);
                _don.Text = conf[5];
                _host.Text = conf[6];
                _mnt.Text = conf[7];
                _pwd.Text = conf[8];
                _don_TextChanged(null, null);
            } catch { }
        }

        /// <summary>
        /// Save config and exit
        /// </summary>
        private void Form1_FormClosing(object sender, FormClosingEventArgs e) {
            System.IO.File.WriteAllLines("foorunes.conf", new string[]{
                "" + _hide.Checked,
                "" + _read.Checked,
                "" + _mp.SelectedIndex,
                "" + _sub.Checked,
                "" + _delay.Checked,
                "" + _don.Text,
                "" + _host.Text,
                "" + _mnt.Text,
                "" + _pwd.Text},
                Encoding.UTF8);
            ni.Dispose();
        }

        /// <summary>
        /// Hide on minimize
        /// </summary>
        private void Form1_Resize(object sender, EventArgs e) {
            if (this.WindowState ==
                FormWindowState.Minimized)
                this.Visible = false;
        }

        /// <summary>
        /// (〜￣▽￣)〜
        /// </summary>
        private void label1_Click(object sender, EventArgs e) {
            System.Diagnostics.Process.Start("http://www.u3m.net/");
        }

        /// <summary>
        /// Update status field
        /// </summary>
        private void q(string s) {
            status.Text = s;
            Application.DoEvents();
        }

        /// <summary>
        /// Copy exception data to clipboard
        /// </summary>
        private void status_Click(object sender, EventArgs e) {
            Clipboard.Clear();
            Clipboard.SetText(trace);
            MessageBox.Show("Exception data copied to clipboard.");
        }
    }
}
